﻿using Sasa.QualityTools.CodeAnalysis.Common;
using System;
using System.Collections.Generic;
using System.IO;
using System.Xml;

namespace Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.Models
{
    public class RuleSet
    {
        public string Name { get; set; }

        public string Path { get; set; }


        public RuleSet()
        {
        }


        public static RuleSet CreateInstance(string path)
        {
            var ruleSet = new RuleSet();
            ruleSet.Name = GetRuleSetName(path);
            ruleSet.Path = path;
            return ruleSet;
        }

        public static IEnumerable<RuleSet> Load(string directory)
        {
            Logger.Info("Loading ruleset files from '{0}'.", directory);

            if (Directory.Exists(directory))
            {
                foreach (string path in Directory.GetFiles(directory, "*.ruleset"))
                {
                    yield return CreateInstance(path);
                }
            }
        }

        internal static string GetRuleSetName(string path)
        {
            try
            {
                var document = new XmlDocument();
                document.Load(path);
                var element = document.SelectSingleNode("/RuleSet/@Name");
                return (element != null) ? element.Value : System.IO.Path.GetFileNameWithoutExtension(path);
            }
            catch (Exception e)
            {
                Logger.Error(e, "Failed to get RuleSet name of '" + path + "'.");
                throw;
            }
        }
    }
}
