﻿using Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.Models;
using System;

namespace Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.ViewModels
{
    public class ProblemViewModel : ViewModelBase
    {
        private Problem problem;

        private string projectName;

        private string fileName;


        public string Id
        {
            get { return problem.Id; }
            set { problem.Id = value; OnPropertyChanged("Id"); }
        }

        public string CheckId
        {
            get { return problem.CheckId; }
            set { problem.CheckId = value; OnPropertyChanged("CheckId"); }
        }

        public string Category
        {
            get { return problem.Category; }
            set { problem.Category = value; OnPropertyChanged("Category"); }
        }

        public string ProjectName
        {
            get { return projectName; }
            set { projectName = value; OnPropertyChanged("ProjectName"); }
        }

        public string Path
        {
            get { return problem.Path; }
            set
            {
                problem.Path = value;
                FileName = (value != null) ? System.IO.Path.GetFileName(value) : null;
                OnPropertyChanged("Path");
            }
        }

        public string FileName
        {
            get { return fileName; }
            private set { fileName = value; OnPropertyChanged("FileName"); }
        }

        public int? Line
        {
            get { return problem.Line; }
            set { problem.Line = value; OnPropertyChanged("Line"); }
        }

        public string Message
        {
            get { return problem.Message; }
            set { problem.Message = value; OnPropertyChanged("Message"); }
        }

        public string AssemblyPath
        {
            get { return problem.AssemblyPath; }
            set { problem.AssemblyPath = value; OnPropertyChanged("AssemblyPath"); }
        }

        public string TypeName
        {
            get { return problem.TypeName; }
            set { problem.TypeName = value; OnPropertyChanged("TypeName"); }
        }

        public string Kind
        {
            get { return Enum.GetName(typeof(CodeKind), problem.Kind); }
            set { problem.Kind = (CodeKind)Enum.Parse(typeof(CodeKind), value); OnPropertyChanged("Kind"); }
        }


        public ProblemViewModel()
            : this(new Problem())
        {
        }

        public ProblemViewModel(Problem problem)
        {
            this.problem = problem;
            FileName = (problem.Path != null) ? System.IO.Path.GetFileName(problem.Path) : null;
        }


        public override string ToString()
        {
            var values = new string[] { Kind.ToString(), CheckId, Category, ProjectName, FileName, Message };
            return String.Join(";", values);
        }
    }
}
