﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Build.Framework;
using Microsoft.Build.Utilities;
using Sasa.QualityTools.CodeAnalysis.Common;

namespace Sasa.QualityTools.CodeAnalysis.MSBuild.Loggers
{
    internal class MSBuildLogger : Sasa.QualityTools.CodeAnalysis.Common.ILogger
    {
        private TaskLoggingHelper logger;

        private List<LogData> logDataCache;


        public IEnumerable<string> Errors
        {
            get
            {
                return logDataCache.Where(o => o.Category == LogCategory.Error).Select(o => o.Message);
            }
        }


        public MSBuildLogger(TaskLoggingHelper logger)
        {
            this.logger = logger;
            logDataCache = new List<LogData>();
        }

        public void Info(string format, params object[] args)
        {
            logDataCache.Add(new LogData { Category = LogCategory.Info, Message = Text.Format(format, args) });
        }

        public void Warn(string format, params object[] args)
        {
            logDataCache.Add(new LogData { Category = LogCategory.Warning, Message = Text.Format(format, args) });
        }

        public void Error(string format, params object[] args)
        {
            logDataCache.Add(new LogData { Category = LogCategory.Error, Message = Text.Format(format, args) });
        }

        public void Error(Exception exception, string format, params object[] args)
        {
            logDataCache.Add(new LogData { Category = LogCategory.Error, Message = Text.Format(format, args) });
            logDataCache.Add(new LogData { Category = LogCategory.Error, Message = Text.Format(exception) });
        }


        private enum LogCategory
        {
            Info,
            Warning,
            Error
        }


        private class LogData
        {
            public LogCategory Category { get; set; }

            public string Message { get; set; }
        }
    }
}
