﻿using Microsoft.Win32;
using Sasa.QualityTools.CodeAnalysis.Common;
using Sasa.QualityTools.CodeAnalysis.Core.Models;
using Sasa.QualityTools.CodeAnalysis.Core.Utilities;
using System;
using System.Globalization;
using System.IO;
using System.Reflection;

namespace Sasa.QualityTools.CodeAnalysis.MSBuild.Tasks
{
    public class PerformCodeMetricsCalculation : PerformCodeAnalysisTaskBase
    {
        private const string RegistryKeyName = @"HKEY_CURRENT_USER\Software\Microsoft\VisualStudio\10.0\DialogPage\Sasa.QualityTools.CodeAnalysis.FxCopIntegrator.Options.GeneralOptionPageData";

        private const string RegistryValueName = "MetricsPath";


        public string MetricsPath { get; set; }


        protected override bool PerformCodeAnalysis()
        {
            if (MetricsPath == null)
            {
                var registryValue = Registry.GetValue(RegistryKeyName, RegistryValueName, null);
                if (registryValue == null)
                {
                    var message = Text.Format("The Metrics.exe path can't be retrieved from the registry ({0}\\{1}).", RegistryKeyName, RegistryValueName);
                    return false;
                }
                MetricsPath = Convert.ToString(registryValue, CultureInfo.InvariantCulture);
            }

            if (!File.Exists(MetricsPath))
            {
                var message = Text.Format("The specified Metrics.exe path ({0}) doesn't exist.", MetricsPath);
                return false;
            }

            var version = AssemblyName.GetAssemblyName(MetricsPath).Version;
            var option = MetricsOptionBuilder.Build(Project, version);

            var calculator = new CodeMetricsCalculator();
            calculator.CommandPath = MetricsPath;
            calculator.CommandOption = option;
            foreach (var r in References)
            {
                calculator.ReferenceAssemblyPaths.Add(r);
            }

            var result = MetricsErrorCodes.None;
            try
            {
                result = calculator.PerformCodeAnalysis();
            }
            catch (CodeAnalysisException e)
            {
                foreach (var log in ErrorLogs)
                {
                    Log.LogError(log);
                }
                Log.LogErrorFromException(e);
                return false;
            }

            return true;
        }
    }
}
